//
//  PLVFloatingWindow.m
//  WebView-Test
//
//  Created by POLYV-UX on 2021/1/15.
//  Copyright © 2021 easefun. All rights reserved.
//

#import "PLVFloatingWindow.h"

#define PLV_ScreenWidth   [UIScreen mainScreen].bounds.size.width
#define PLV_ScreenHeight  [UIScreen mainScreen].bounds.size.height

@interface PLVFloatingWindow ()<UIGestureRecognizerDelegate>

/// 悬浮窗宽度，默认值为设备屏幕宽度0.3
@property (nonatomic, assign) CGFloat windowWidth;
/// 悬浮窗宽高比，默认 3 : 4
@property (nonatomic, assign) CGFloat sizeScale;
/// 悬浮窗初始位置，默认为离屏幕底部 100pt，离屏幕右侧 10pt
@property (nonatomic, assign) CGPoint originPoint;


@property (nonatomic, strong) WKWebView *mainWebview;
@property (nonatomic, strong) UIButton *closeButton;
@property (nonatomic, strong) UITapGestureRecognizer *tapGestureRecognizer;
@property (nonatomic, strong) UIPanGestureRecognizer *panGestureRecognizer;


@property (nonatomic, assign) CGRect originalFrame; //!< webView在原控制器的位置大小
@property (nonatomic, strong) PLVFloatingWKWebViewController *originalViewController;   //!< webview原来所在的控制器
@property (nonatomic, strong) UINavigationController *originalNavigation;   //!< webview原来所在的控制器的导航

@end


@implementation PLVFloatingWindow

#pragma mark - Init

-(instancetype)init
{
    if (self = [super init]) {
        self.hidden = YES;
        self.windowLevel = UIWindowLevelNormal + 1;
        self.backgroundColor = [UIColor whiteColor];
        //默认的小窗位置和大小
        self.sizeScale = 3.0 / 4.0;
        self.windowWidth = MIN(PLV_ScreenWidth, PLV_ScreenHeight) * 0.3;
        [self resetFrame];
        
        [self addGestureRecognizer:self.tapGestureRecognizer];
        [self addGestureRecognizer:self.panGestureRecognizer];
        [self addSubview:self.closeButton];
        
    }
    return self;
}

+(instancetype)sharedInstance
{
    static PLVFloatingWindow *_sharedInstance = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        _sharedInstance = [self new];
    });
    return _sharedInstance;
}

#pragma mark - Action
#pragma mark - Public Methods

/// 把webview从控制器移动到window上
/// @param webview 要移动的webview
/// @param height webview的内容高度（为0则使用默认高度）
/// @param containerVC webview所在的控制器
-(void)moveWebviewToWindow:(WKWebView *)webview contentHeight:(CGFloat)height fromController:(PLVFloatingWKWebViewController *)containerVC
{
    // 若没有获取到webview内容高度，则给定一个默认高度
    CGFloat contentHeight = height > 0 ? height : 603;
    
    self.sizeScale = PLV_ScreenWidth / contentHeight;
    [self resetFrame];
    
    self.mainWebview = webview;
    self.originalFrame = self.mainWebview.frame;
    self.mainWebview.frame = CGRectMake(0, 0, self.frame.size.width, self.frame.size.height);
    self.mainWebview.scrollView.userInteractionEnabled = NO;
    
    self.originalViewController = containerVC;
    self.originalNavigation = containerVC.navigationController;
    
    [self insertSubview:self.mainWebview atIndex:0];
    self.hidden = NO;
}


#pragma mark - Private Methods

/// 重设window的位置大小
-(void)resetFrame
{
    CGFloat windowHeight = self.windowWidth / self.sizeScale;
    self.originPoint = CGPointMake(PLV_ScreenWidth - self.windowWidth - 10, PLV_ScreenHeight - windowHeight - 100);
    self.frame = CGRectMake(self.originPoint.x, self.originPoint.y, self.windowWidth, windowHeight);
    self.closeButton.frame = CGRectMake(self.frame.size.width - 35, 0, 35, 35);
}

/// 处理关闭小窗口
-(void)clickCloseWindowAction
{
    self.originalViewController = nil;
    self.originalNavigation = nil;
    [self.mainWebview stopLoading];
    [self.mainWebview removeFromSuperview];
    self.mainWebview = nil;
    self.hidden = YES;
}

/// 源控制器存在于导航栈的情况下，处理把webview还原到原控制器
-(void)dealRecoverWebView
{
    if (self.recoverWebViewBlock) {
        self.originalViewController = nil;
        self.originalNavigation = nil;
        //还原frame
        self.mainWebview.frame = self.originalFrame;
        self.mainWebview.scrollView.userInteractionEnabled = YES;
        self.hidden = YES;
        self.recoverWebViewBlock(self.mainWebview);
        self.mainWebview = nil;
    }
}

/// 原控制器不在，处理恢复webview到重建控制器
-(void)dealRebuildWebView
{
    self.mainWebview.frame = self.originalFrame;
    self.mainWebview.scrollView.userInteractionEnabled = YES;
    self.hidden = YES;
    PLVFloatingWKWebViewController *resetContainerViewController = [[PLVFloatingWKWebViewController alloc]initWithWebView:self.mainWebview];
    [self pushResetContainerVC:resetContainerViewController];
}


/// push到新建的控制器
/// @param resetContainerViewController 新建的控制器
-(void)pushResetContainerVC:(PLVFloatingWKWebViewController *)resetContainerViewController
{
    UIViewController *rootViewController = [UIApplication sharedApplication].delegate.window.rootViewController;
    if ([rootViewController isKindOfClass:[UINavigationController class]]) {
        UINavigationController *rootNavigation = (UINavigationController *)rootViewController;
        if ([rootNavigation presentedViewController]) {
            //如果有present其他页面，则需要先dismiss
            UIViewController *presentedVC = [rootNavigation presentedViewController];
            [presentedVC dismissViewControllerAnimated:NO completion:^{
                [rootNavigation pushViewController:resetContainerViewController animated:YES];
            }];
        } else {
            [rootNavigation pushViewController:resetContainerViewController animated:YES];
        }
    } else {
        UINavigationController *resetNavigation = [[UINavigationController alloc] initWithRootViewController:resetContainerViewController];
        [rootViewController presentViewController:resetNavigation animated:YES completion:nil];
    }
}

#pragma mark - Gesture

/// 处理单击手势，还原小窗
/// @param gesture gesture
-(void)tapWindow:(UITapGestureRecognizer *)gesture
{
    if (self.originalNavigation) {
        //通过导航方式加载的控制器
        NSArray *vcArray = self.originalNavigation.viewControllers;
        NSInteger index = [vcArray indexOfObject:self.originalViewController];
        if (index == NSNotFound) {
            //原控制器已经不存在，则新建控制器
            [self dealRebuildWebView];
        }else {
            if (index == vcArray.count - 1) {
                // 原控制器在导航栈顶
                [self dealRecoverWebView];
            }else {
                // 原控制器不在导航栈顶
                [self.originalNavigation popToViewController:self.originalViewController animated:NO];
                [self dealRecoverWebView];
            }
        }
    }
    else {
        // 不是通过导航方式加载的控制器
        [self dealRecoverWebView];
    }
}

/// 拖拽窗口
-(void)dragWindow:(UIPanGestureRecognizer *)gesture
{
    CGPoint translatedPoint = [gesture translationInView:[UIApplication sharedApplication].delegate.window];
    CGFloat x = gesture.view.center.x + translatedPoint.x;
    CGFloat y = gesture.view.center.y + translatedPoint.y;
    
    if (gesture.state == UIGestureRecognizerStateBegan || gesture.state == UIGestureRecognizerStateChanged) {
        gesture.view.center = CGPointMake(x, y);
        [gesture setTranslation:CGPointMake(0, 0) inView:[UIApplication sharedApplication].delegate.window];
        return;
    }
    
    CGFloat navigationHeight = [[UIApplication sharedApplication] statusBarFrame].size.height + 44;
    CGFloat width = self.frame.size.width;
    CGFloat height = self.frame.size.height;
    
    // 不允许拖离屏幕
    if (x > PLV_ScreenWidth - 0.5 * width) {
        x = PLV_ScreenWidth - 0.5 * width;
    } else if (x < width * 0.5) {
        x = width * 0.5;
    }
    
    if (y > PLV_ScreenHeight - height * 0.5) {
        y = PLV_ScreenHeight - height * 0.5;
    } else if (y < height * 0.5 + navigationHeight) {
        y = height * 0.5 + navigationHeight;
    }
    
    gesture.view.center = CGPointMake(x, y);
    [gesture setTranslation:CGPointMake(0, 0) inView:[UIApplication sharedApplication].delegate.window];
}

#pragma mark - UIGestureRecognizerDelegate

-(BOOL)gestureRecognizer:(UIGestureRecognizer *)gestureRecognizer shouldRecognizeSimultaneouslyWithGestureRecognizer:(UIGestureRecognizer *)otherGestureRecognizer
{
    return YES;
}


#pragma mark - Loadlazy

-(UIButton *)closeButton
{
    if (!_closeButton) {
        _closeButton = [UIButton buttonWithType:UIButtonTypeCustom];
        [_closeButton setImage:[UIImage imageNamed:@"plv_floating_btn_close"] forState:0];
        _closeButton.backgroundColor = [UIColor lightGrayColor];
        [_closeButton addTarget:self action:@selector(clickCloseWindowAction) forControlEvents:UIControlEventTouchUpInside];
    }
    return _closeButton;
}

- (UITapGestureRecognizer *)tapGestureRecognizer
{
    if (!_tapGestureRecognizer) {
        _tapGestureRecognizer = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(tapWindow:)];
        _tapGestureRecognizer.delegate = self;
    }
    return _tapGestureRecognizer;
}

- (UIPanGestureRecognizer *)panGestureRecognizer {
    if (!_panGestureRecognizer) {
        _panGestureRecognizer = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(dragWindow:)];
        _panGestureRecognizer.delegate = self;
    }
    return _panGestureRecognizer;
}

@end
