//
//  PLVFloatingWKWebViewController.m
//  WebView-Test
//
//  Created by POLYV-UX on 2021/1/15.
//  Copyright © 2021 easefun. All rights reserved.
//

#import "PLVFloatingWKWebViewController.h"
#import "PLVFloatingWindow.h"
#import "PLVUtil.h"

@interface PLVFloatingWKWebViewController ()<WKNavigationDelegate, WKUIDelegate, WKScriptMessageHandler>
@property (nonatomic, strong) NSString *urlString;  //!< 加载的网页链接

@property (nonatomic, strong) WKWebView *mainWebview;
@property (nonatomic, strong) WKWebView *otherWebview;

@property (nonatomic, assign) BOOL webviewLoadFinish;
@property (nonatomic, assign) CGFloat webViewContentHeight;

@end

@implementation PLVFloatingWKWebViewController

#pragma mark - Lift cycle & dealloc

-(void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    [self.mainWebview.configuration.userContentController addScriptMessageHandler:self name:@"changeToSmall"];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    [PLVUtil rotateOrientation:UIInterfaceOrientationPortrait];
    [self setUpUI];
}

-(void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    [self.mainWebview.configuration.userContentController removeScriptMessageHandlerForName:@"changeToSmall"];
}

-(void)dealloc
{
    NSLog(@"dealloc");
}


#pragma mark - Init & UI

/// 初始化控制器
/// @param urlString 网页url路径
-(instancetype)initWithUrlString:(NSString *)urlString
{
    if (self = [super init]) {
        _urlString = urlString;
    }
    return self;
}

/// 当存在悬浮窗，但原来webview的容器控制器不存在，通过此方法重建容器
/// @param webview webview
-(instancetype)initWithWebView:(WKWebView *)webview
{
    if (self = [super init]) {
        self.mainWebview = webview;
    }
    return self;
}

-(void)setUpUI
{
    self.title = @"可小窗 WKWebView";
    self.view.backgroundColor = [UIColor lightTextColor];
    self.edgesForExtendedLayout = UIRectEdgeNone;
    
    if (self.mainWebview) {
        //通过webview重建控制器
        
        // 1.关闭小窗
        [[PLVFloatingWindow sharedInstance] clickCloseWindowAction];
        // 2.告诉h5还原小窗布局
        [self mainWebviewCallH5Function:@"revertToBig"];
        
        self.mainWebview.scrollView.userInteractionEnabled = YES;
        self.mainWebview.frame = self.view.bounds;
        [self.view addSubview:self.mainWebview];
    }else {
        //通过url新建控制器
        WKUserContentController *userContentController = [[WKUserContentController alloc] init];
        WKWebViewConfiguration *config = [[WKWebViewConfiguration alloc] init];
        config.userContentController = userContentController;
        config.allowsInlineMediaPlayback = YES;
        config.requiresUserActionForMediaPlayback = NO;
        
        self.mainWebview = [[WKWebView alloc] initWithFrame:self.view.bounds configuration:config];
        self.mainWebview.navigationDelegate = self;
        self.mainWebview.UIDelegate = self;
        [self.view addSubview:self.mainWebview];
        
        if (self.urlString.length) {
            NSURL *h5Url = [NSURL URLWithString:self.urlString];
            NSURLRequest *request = [NSURLRequest requestWithURL:h5Url];
            [self.mainWebview loadRequest:request];
        }
    }
    
    WKWebViewConfiguration *config = [[WKWebViewConfiguration alloc] init];
    config.allowsInlineMediaPlayback = YES;
    config.requiresUserActionForMediaPlayback = NO;
    CGRect otherFrame = CGRectMake(self.view.bounds.size.width, self.mainWebview.frame.origin.y, self.view.bounds.size.width, self.view.bounds.size.height);
    
    self.otherWebview = [[WKWebView alloc] initWithFrame:otherFrame configuration:config];
    self.otherWebview.backgroundColor = [UIColor whiteColor];
    self.otherWebview.navigationDelegate = self;
    self.otherWebview.UIDelegate = self;
    [self.view addSubview:self.otherWebview];
    
    //设置回调，处理还原小窗
    __weak typeof(self) weakSelf = self;
    [[PLVFloatingWindow sharedInstance] setRecoverWebViewBlock:^(WKWebView * _Nonnull webview) {
        [weakSelf handleRecoverWebView:webview];
    }];
}

#pragma mark - Action

/// 处理h5的changeToSmall事件，打开小窗
/// @param bodyDict h5传递的参数，包含要打开的h5的链接url
-(void)openFloatingWindowAction:(NSDictionary *)bodyDict
{
    // 1.使用otherWebview打开新的h5页面
    NSString *url = bodyDict[@"url"];
    NSURLRequest *request = [NSURLRequest requestWithURL:[NSURL URLWithString:url]];
    [self.otherWebview loadRequest:request];
    
    // 2.把otherWebview移动到屏幕上
    [self.otherWebview.layer addAnimation:[self moveOtherWebviewAnimation] forKey:@"move_webview"];
    
    // 3.把mainWebview移动到window上面作为小窗存在
    [[PLVFloatingWindow sharedInstance] moveWebviewToWindow:self.mainWebview contentHeight:self.webViewContentHeight fromController:self];
}

/// 移动OtherWebview的动画
- (CAAnimation *)moveOtherWebviewAnimation{
    CGPoint startPoint = self.otherWebview.center;
    CGPoint endPoint = self.mainWebview.center;
    CABasicAnimation *animation=[CABasicAnimation animationWithKeyPath:@"position"];
    animation.fromValue = [NSValue valueWithCGPoint:startPoint];
    animation.toValue = [NSValue valueWithCGPoint:endPoint];
    animation.duration = 0.45;
    animation.removedOnCompletion = NO;
    animation.fillMode = kCAFillModeForwards;
    animation.timingFunction =
        [CAMediaTimingFunction functionWithName: kCAMediaTimingFunctionEaseInEaseOut];
    return animation;
}

/// 处理webview还原
/// @param webview mainWebview
-(void)handleRecoverWebView:(WKWebView *)webview
{
    // 1.告诉h5还原小窗布局
    [self mainWebviewCallH5Function:@"revertToBig"];
    // 2.把otherWebview移动到屏幕之外的原位置
    [self.otherWebview.layer removeAnimationForKey:@"move_webview"];
    // 3.把window上面的mainWebview移动回屏幕上
    [self.view addSubview:webview];
}

/// mainWebview调用H5的js方法
/// @param jsFunction js方法名称
-(void)mainWebviewCallH5Function:(NSString *)jsFunction
{
    __weak typeof(self) weakSelf = self;
    if (!self.webviewLoadFinish) {
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
            [weakSelf mainWebviewCallH5Function:jsFunction];
        });
    }else{
        NSString *jsStr = [NSString stringWithFormat:@"%@()",jsFunction];
        [self.mainWebview evaluateJavaScript:jsStr completionHandler:^(id _Nullable data, NSError * _Nullable error) {
            if (error) {
                NSLog(@"PLVJS - OC调用%@失败，%@",jsFunction,error.description);
            }
        }];
    }
}


#pragma mark - WKScriptMessageHandler
-(void)userContentController:(WKUserContentController *)userContentController didReceiveScriptMessage:(WKScriptMessage *)message
{
    if (message.webView == self.mainWebview) {
        if ([message.name isEqualToString:@"changeToSmall"]) {
            //打开小窗，并加载新的url
            [self openFloatingWindowAction:message.body];
        }
    }
}

#pragma mark - WKNavigationDelegate
-(void)webView:(WKWebView *)webView didFinishNavigation:(WKNavigation *)navigation
{
    if (webView == self.mainWebview) {
        self.webviewLoadFinish = YES;
        //获取webview的内容高度，用于等比例缩放为小窗尺寸
        __weak typeof(self) weakSelf = self;
        [webView evaluateJavaScript:@"document.documentElement.scrollHeight" completionHandler:^(id result, NSError *_Nullable error) {
            weakSelf.webViewContentHeight = [result floatValue];
        }];
    }
}

-(void)webView:(WKWebView *)webView decidePolicyForNavigationAction:(WKNavigationAction *)navigationAction decisionHandler:(void (^)(WKNavigationActionPolicy))decisionHandler
{
    NSLog(@"%@",navigationAction.request.URL.absoluteString);
    //允许跳转
    decisionHandler(WKNavigationActionPolicyAllow);
}


#pragma mark - UIViewController+UIViewControllerRotation

- (UIInterfaceOrientationMask)supportedInterfaceOrientations {
    return UIInterfaceOrientationMaskPortrait;
}

- (BOOL)shouldAutorotate {
    return YES;
}


/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
